@@ -1,6 +1,6 @@
 /* -*-mode: c; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-*/
 /*
-** Copyright (C) 2008-2013 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+** Copyright (C) 2008-2014 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -30,10 +30,6 @@
 #include "mu-msg-priv.h"
 #include "mu-msg-part.h"
 
-static gboolean handle_children (MuMsg *msg,
-				 GMimeMessage *mime_msg, MuMsgOptions opts,
-				 unsigned index, MuMsgPartForeachFunc func,
-				 gpointer user_data);
 struct _DoData {
 	GMimeObject *mime_obj;
 	unsigned    index;
@@ -46,8 +42,13 @@ do_it_with_index (MuMsg *msg, MuMsgPart *part, DoData *ddata)
 	if (ddata->mime_obj)
 		return;
 
-	if (part->index == ddata->index)
+	if (part->index == ddata->index) {
+		/* Add a reference to this object, this way if it is
+		 * encrypted it will not be garbage collected before
+		 * we are done with it. */
+		g_object_ref (part->data);
 		ddata->mime_obj = (GMimeObject*)part->data;
+	}
 }
 
 static GMimeObject*
@@ -151,16 +152,28 @@ accumulate_text (MuMsg *msg, MuMsgPart *part, GString **gstrp)
 		accumulate_text_part (msg, part, gstrp);
 }
 
+/* declaration, so we can use it earlier */
+static gboolean
+handle_mime_object (MuMsg *msg, GMimeObject *mobj, GMimeObject *parent,
+                    MuMsgOptions opts, unsigned *index, gboolean decrypted,
+                    MuMsgPartForeachFunc func, gpointer user_data);
+
 static char*
-get_text_from_mime_msg (MuMsg *msg, GMimeMessage *mmsg, MuMsgOptions opts,
-			unsigned index)
+get_text_from_mime_msg (MuMsg *msg, GMimeMessage *mmsg, MuMsgOptions opts)
 {
 	GString *gstr;
+	unsigned index;
 
-	gstr = g_string_sized_new (4096);
-	handle_children (msg, mmsg, opts, index,
-			 (MuMsgPartForeachFunc)accumulate_text,
-			 &gstr);
+	index = 1;
+	gstr  = g_string_sized_new (4096);
+	handle_mime_object (msg,
+	                    mmsg->mime_part,
+			    (GMimeObject *) mmsg,
+			    opts,
+			    &index,
+	                    FALSE,
+			    (MuMsgPartForeachFunc)accumulate_text,
+			    &gstr);
 
 	return g_string_free (gstr, FALSE);
 }
@@ -174,7 +187,8 @@ mu_msg_part_get_text (MuMsg *msg, MuMsgPart *self, MuMsgOptions opts)
 	gboolean err;
 
 	g_return_val_if_fail (msg, NULL);
-	g_return_val_if_fail (self && self->data, NULL);
+	g_return_val_if_fail (self && GMIME_IS_OBJECT(self->data),
+			      NULL);
 
 	mobj = (GMimeObject*)self->data;
 
@@ -198,8 +212,7 @@ mu_msg_part_get_text (MuMsg *msg, MuMsgPart *self, MuMsgOptions opts)
 	/* apparently, g_mime_message_part_get_message may still
 	 * return NULL */
  	if (mime_msg)
-		return get_text_from_mime_msg (msg, mime_msg,
-					       opts, self->index);
+		return get_text_from_mime_msg (msg, mime_msg, opts);
 	return NULL;
 }
 
@@ -303,47 +316,18 @@ get_disposition (GMimeObject *mobj)
 	return MU_MSG_PART_TYPE_NONE;
 }
 
-/* declaration, so we can use it in handle_encrypted_part */
-static gboolean handle_mime_object (MuMsg *msg,
-				    GMimeObject *mobj, GMimeObject *parent,
-				    MuMsgOptions opts,
-				    unsigned index, MuMsgPartForeachFunc func,
-				    gpointer user_data);
-
-/* if mu is build without crypto support (i.e, gmime < 2.6), the crypto funcs
- * are no-ops
- */
-#ifndef BUILD_CRYPTO
-
-#define check_signature(A,B,C) (TRUE)
-#define handle_encrypted_part(A,B,C,D,E,F,G) (TRUE)
-
-#else /* BUILD_CRYPTO */
-
-#define SIG_STATUS_REPORT "sig-status-report"
-
 /* call 'func' with information about this MIME-part */
-static gboolean
+static inline void
 check_signature (MuMsg *msg, GMimeMultipartSigned *part, MuMsgOptions opts)
 {
-	/* the signature status */
-	MuMsgPartSigStatusReport *sigrep;
 	GError *err;
 
-	err     = NULL;
-	sigrep = mu_msg_crypto_verify_part (part, opts, &err);
+	err = NULL;
+	mu_msg_crypto_verify_part (part, opts, &err);
 	if (err) {
 		g_warning ("error verifying signature: %s", err->message);
 		g_clear_error (&err);
 	}
-
-	/* tag this part with the signature status check */
-	g_object_set_data_full
-		(G_OBJECT(part), SIG_STATUS_REPORT,
-		 sigrep,
-		 (GDestroyNotify)mu_msg_part_sig_status_report_destroy);
-
-	return TRUE;
 }
 
 
@@ -379,12 +363,12 @@ get_console_pw (const char* user_id, const char *prompt_ctx,
 
 
 static gboolean
-handle_encrypted_part (MuMsg *msg,
-		       GMimeMultipartEncrypted *part, GMimeObject *parent,
-		       MuMsgOptions opts, unsigned index,
+handle_encrypted_part (MuMsg *msg, GMimeMultipartEncrypted *part,
+		       MuMsgOptions opts, unsigned *index,
 		       MuMsgPartForeachFunc func, gpointer user_data)
 {
 	GError *err;
+	gboolean rv;
 	GMimeObject *dec;
 	MuMsgPartPasswordFunc pw_func;
 
@@ -402,22 +386,33 @@ handle_encrypted_part (MuMsg *msg,
 	}
 
 	if (dec) {
-		gboolean rv;
-		rv = handle_mime_object (msg, dec, parent, opts,
-					 index + 1, func, user_data);
+		rv = handle_mime_object (msg, dec, (GMimeObject *) part,
+		                         opts, index, TRUE, func, user_data);
 		g_object_unref (dec);
-		return rv;
+	} else {
+		/* On failure to decrypt, list the encrypted part as
+		 * an attachment
+		 */
+		GMimeObject *encrypted;
+
+		encrypted = g_mime_multipart_get_part (
+			GMIME_MULTIPART (part), 1);
+
+		g_return_val_if_fail (GMIME_IS_PART(encrypted), FALSE);
+
+		rv = handle_mime_object (msg, encrypted, (GMimeObject *) part,
+		                         opts, index, FALSE, func, user_data);
 	}
 
-	return TRUE;
+	return rv;
 }
-#endif /*BUILD_CRYPTO */
+
 
 
 /* call 'func' with information about this MIME-part */
 static gboolean
 handle_part (MuMsg *msg, GMimePart *part, GMimeObject *parent,
-	     MuMsgOptions opts, unsigned index,
+	     MuMsgOptions opts, unsigned *index, gboolean decrypted,
 	     MuMsgPartForeachFunc func, gpointer user_data)
 {
 	GMimeContentType *ct;
@@ -427,7 +422,13 @@ handle_part (MuMsg *msg, GMimePart *part, GMimeObject *parent,
 
 	msgpart.size        = get_part_size (part);
 	msgpart.part_type   = MU_MSG_PART_TYPE_LEAF;
-	msgpart.part_type |= get_disposition ((GMimeObject*)part);
+	msgpart.part_type  |= get_disposition ((GMimeObject*)part);
+	if (decrypted)
+		msgpart.part_type |= MU_MSG_PART_TYPE_DECRYPTED;
+	else if ((opts & MU_MSG_OPTION_DECRYPT) &&
+	         GMIME_IS_MULTIPART_ENCRYPTED (parent))
+		msgpart.part_type |= MU_MSG_PART_TYPE_ENCRYPTED;
+
 
 	ct = g_mime_object_get_content_type ((GMimeObject*)part);
 	if (GMIME_IS_CONTENT_TYPE(ct)) {
@@ -441,17 +442,22 @@ handle_part (MuMsg *msg, GMimePart *part, GMimeObject *parent,
 			msgpart.part_type |= MU_MSG_PART_TYPE_TEXT_HTML;
 	}
 
-#ifdef BUILD_CRYPTO
-	/* put the verification info in the pgp-signature part */
+	/* put the verification info in the pgp-signature and every
+	 * descendent of a pgp-encrypted part */
 	msgpart.sig_status_report = NULL;
-	if (g_ascii_strcasecmp (msgpart.subtype, "pgp-signature") == 0)
+	if (g_ascii_strcasecmp (msgpart.subtype, "pgp-signature") == 0 ||
+	    decrypted) {
 		msgpart.sig_status_report =
 			(MuMsgPartSigStatusReport*)
-			g_object_get_data (G_OBJECT(parent), SIG_STATUS_REPORT);
-#endif /*BUILD_CRYPTO*/
+			g_object_get_data (G_OBJECT(parent),
+					   SIG_STATUS_REPORT);
+
+		if (msgpart.sig_status_report)
+			msgpart.part_type |= MU_MSG_PART_TYPE_SIGNED;
+	}
 
 	msgpart.data    = (gpointer)part;
-	msgpart.index   = index;
+	msgpart.index   = (*index)++;
 
 	func (msg, &msgpart, user_data);
 
@@ -461,8 +467,9 @@ handle_part (MuMsg *msg, GMimePart *part, GMimeObject *parent,
 
 /* call 'func' with information about this MIME-part */
 static gboolean
-handle_message_part (MuMsg *msg, GMimeMessagePart *mimemsgpart, GMimeObject *parent,
-		     MuMsgOptions opts, unsigned index,
+handle_message_part (MuMsg *msg, GMimeMessagePart *mimemsgpart,
+		     GMimeObject *parent, MuMsgOptions opts, unsigned *index,
+		     gboolean decrypted,
 		     MuMsgPartForeachFunc func, gpointer user_data)
 {
 	MuMsgPart msgpart;
@@ -471,7 +478,7 @@ handle_message_part (MuMsg *msg, GMimeMessagePart *mimemsgpart, GMimeObject *par
 
 	msgpart.type        = "message";
 	msgpart.subtype     = "rfc822";
-	msgpart.index       = index;
+	msgpart.index       = (*index)++;
 
 	/* msgpart.size        = 0; /\* maybe calculate this? *\/ */
 
@@ -482,84 +489,73 @@ handle_message_part (MuMsg *msg, GMimeMessagePart *mimemsgpart, GMimeObject *par
 	func (msg, &msgpart, user_data);
 
 	if (opts & MU_MSG_OPTION_RECURSE_RFC822) {
-		GMimeMessage *mmsg; /* this may return NULL for some messages */
+		GMimeMessage *mmsg; /* may return NULL for some
+				     * messages */
 		mmsg = g_mime_message_part_get_message (mimemsgpart);
 		if (mmsg)
-			return handle_children
-				(msg, mmsg,
-				 opts, index, func, user_data);
+			return handle_mime_object (msg,
+			                           mmsg->mime_part,
+			                           parent,
+			                           opts,
+			                           index,
+			                           decrypted,
+			                           func,
+			                           user_data);
 	}
 
 	return TRUE;
 }
 
+static gboolean
+handle_multipart (MuMsg *msg, GMimeMultipart *mpart, GMimeObject *parent,
+                  MuMsgOptions opts, unsigned *index, gboolean decrypted,
+		  MuMsgPartForeachFunc func, gpointer user_data)
+{
+	gboolean res;
+	GMimeObject *part;
+	guint i;
+
+	res = TRUE;
+	for (i = 0; i < mpart->children->len; i++) {
+		part = (GMimeObject *) mpart->children->pdata[i];
+		res &= handle_mime_object (msg, part, parent,
+		                           opts, index, decrypted,
+		                           func, user_data);
+	}
+
+	return res;
+}
+
 
 static gboolean
-handle_mime_object (MuMsg *msg,
-		    GMimeObject *mobj, GMimeObject *parent, MuMsgOptions opts,
-		    unsigned index, MuMsgPartForeachFunc func, gpointer user_data)
+handle_mime_object (MuMsg *msg, GMimeObject *mobj, GMimeObject *parent,
+		    MuMsgOptions opts, unsigned *index, gboolean decrypted,
+		    MuMsgPartForeachFunc func, gpointer user_data)
 {
 	if (GMIME_IS_PART (mobj))
 		return handle_part
 			(msg, GMIME_PART(mobj), parent,
-			 opts, index, func, user_data);
+			 opts, index, decrypted, func, user_data);
 	else if (GMIME_IS_MESSAGE_PART (mobj))
 		return handle_message_part
 			(msg, GMIME_MESSAGE_PART(mobj),
-			 parent, opts, index, func, user_data);
+			 parent, opts, index, decrypted, func, user_data);
 	else if ((opts & MU_MSG_OPTION_VERIFY) &&
-		 GMIME_IS_MULTIPART_SIGNED (mobj))
-		return check_signature
+	         GMIME_IS_MULTIPART_SIGNED (mobj)) {
+		check_signature
 			(msg, GMIME_MULTIPART_SIGNED (mobj), opts);
-	else if ((opts & MU_MSG_OPTION_DECRYPT) &&
-		GMIME_IS_MULTIPART_ENCRYPTED (mobj))
+		return handle_multipart
+			(msg, GMIME_MULTIPART (mobj), mobj, opts,
+			 index, decrypted, func, user_data);
+	} else if ((opts & MU_MSG_OPTION_DECRYPT) &&
+	           GMIME_IS_MULTIPART_ENCRYPTED (mobj))
 		return handle_encrypted_part
 			(msg, GMIME_MULTIPART_ENCRYPTED (mobj),
-			 parent, opts, index, func, user_data);
-	return TRUE;
-}
-
-struct _ForeachData {
-	MuMsgPartForeachFunc func;
-	gpointer             user_data;
-	MuMsg                *msg;
-	unsigned             index;
-	MuMsgOptions         opts;
-
-};
-typedef struct _ForeachData ForeachData;
-
-static void
-each_child (GMimeObject *parent, GMimeObject *part,
-	    ForeachData *fdata)
-{
-	handle_mime_object (fdata->msg,
-			    part,
-			    parent,
-			    fdata->opts,
-			    fdata->index++,
-			    fdata->func,
-			    fdata->user_data);
-}
-
-
-static gboolean
-handle_children (MuMsg *msg,
-		 GMimeMessage *mime_msg, MuMsgOptions opts,
-		 unsigned index, MuMsgPartForeachFunc func,
-		 gpointer user_data)
-{
-	ForeachData fdata;
-
- 	fdata.func	= func;
-	fdata.user_data = user_data;
-	fdata.opts	= opts;
-	fdata.msg	= msg;
-	fdata.index	= 0;
-
-	g_mime_message_foreach (mime_msg, (GMimeObjectForeachFunc)each_child,
-				&fdata);
-
+			 opts, index, func, user_data);
+	else if (GMIME_IS_MULTIPART (mobj))
+		return handle_multipart
+			(msg, GMIME_MULTIPART (mobj), parent, opts,
+			 index, decrypted, func, user_data);
 	return TRUE;
 }
 
@@ -568,13 +564,22 @@ gboolean
 mu_msg_part_foreach (MuMsg *msg, MuMsgOptions opts,
 		     MuMsgPartForeachFunc func, gpointer user_data)
 {
+	unsigned index;
+
+	index = 1;
 	g_return_val_if_fail (msg, FALSE);
 
 	if (!mu_msg_load_msg_file (msg, NULL))
 		return FALSE;
 
-	return handle_children (msg, msg->_file->_mime_msg,
-				opts, 0, func, user_data);
+	return handle_mime_object (msg,
+	                           msg->_file->_mime_msg->mime_part,
+	                           (GMimeObject *) msg->_file->_mime_msg,
+	                           opts,
+	                           &index,
+	                           FALSE,
+	                           func,
+	                           user_data);
 }
 
 
@@ -610,7 +615,7 @@ write_part_to_fd (GMimePart *part, int fd, GError **err)
 	} else
 		rv = TRUE;
 
-	g_object_unref (wrapper);
+	/* g_object_unref (wrapper); we don't own it */
 	g_object_unref (stream);
 
 	return rv;
@@ -671,6 +676,10 @@ save_object (GMimeObject *obj, MuMsgOptions opts, const char *fullpath,
 	else
 		rv = write_object_to_fd (obj, fd, err);
 
+	/* Unref it since it was referenced earlier by
+	 * get_mime_object_at_index */
+	g_object_unref (obj);
+
 	if (close (fd) != 0 && !err) { /* don't write on top of old err */
 		g_set_error (err, MU_ERROR_DOMAIN, MU_ERROR_FILE,
 			     "could not close '%s': %s",
@@ -704,6 +713,10 @@ mu_msg_part_get_path (MuMsg *msg, MuMsgOptions opts,
 	fname = mime_part_get_filename (mobj, index, TRUE);
 	filepath = g_build_path (G_DIR_SEPARATOR_S, targetdir ? targetdir : "",
 				 fname, NULL);
+
+	/* Unref it since it was referenced earlier by
+	 * get_mime_object_at_index */
+	g_object_unref (mobj);
 	g_free (fname);
 
 	return filepath;
@@ -781,7 +794,8 @@ mu_msg_part_save (MuMsg *msg, MuMsgOptions opts,
 
 
 gchar*
-mu_msg_part_save_temp (MuMsg *msg, MuMsgOptions opts, guint partidx, GError **err)
+mu_msg_part_save_temp (MuMsg *msg, MuMsgOptions opts, guint partidx,
+		       GError **err)
 {
 	gchar *filepath;
 
@@ -808,7 +822,8 @@ match_cid (MuMsgPart *mpart, const char *cid)
 }
 
 int
-mu_msg_find_index_for_cid (MuMsg *msg, MuMsgOptions opts, const char *sought_cid)
+mu_msg_find_index_for_cid (MuMsg *msg, MuMsgOptions opts,
+			   const char *sought_cid)
 {
 	const char* cid;
 
