@@ -22,7 +22,7 @@
 #include "config.h"
 #endif /*HAVE_CONFIG_H*/
 
-#define _BSD_SOURCE
+#define _GNU_SOURCE
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -258,7 +258,7 @@ get_docid_from_msgid (MuQuery *query, const char *str, GError **err)
 	unsigned docid;
 	MuMsgIter *iter;
 
-	querystr = g_strdup_printf ("msgid:%s", str);
+	querystr = g_strdup_printf ("msgid:\"%s\"", str);
 	iter = mu_query_run (query, querystr,
 			     MU_MSG_FIELD_ID_NONE,
 			     1, MU_QUERY_FLAG_NONE, err);
@@ -293,7 +293,7 @@ get_docids_from_msgids (MuQuery *query, const char *str, GError **err)
 	MuMsgIter *iter;
 	GSList *lst;
 
-	querystr = g_strdup_printf ("msgid:%s", str);
+	querystr = g_strdup_printf ("msgid:\"%s\"", str);
 	iter = mu_query_run (query, querystr, MU_MSG_FIELD_ID_NONE,
 			     -1 /*unlimited*/, MU_QUERY_FLAG_NONE,
 			     err);
@@ -415,8 +415,14 @@ cmd_add (ServerContext *ctx, GHashTable *args, GError **err)
 }
 
 
+struct _PartInfo {
+	GSList      *attlist;
+	MuMsgOptions opts;
+};
+typedef struct _PartInfo PartInfo;
+
 static void
-each_part (MuMsg *msg, MuMsgPart *part, GSList **attlist)
+each_part (MuMsg *msg, MuMsgPart *part, PartInfo *pinfo)
 {
 	char *att, *cachefile;
 	GError *err;
@@ -427,7 +433,8 @@ each_part (MuMsg *msg, MuMsgPart *part, GSList **attlist)
 		return;
 
 	err	  = NULL;
-	cachefile = mu_msg_part_save_temp (msg, MU_MSG_OPTION_OVERWRITE,
+	cachefile = mu_msg_part_save_temp (msg,
+					   pinfo->opts|MU_MSG_OPTION_OVERWRITE,
 					   part->index, &err);
 	if (!cachefile) {
 		print_and_clear_g_error (&err);
@@ -437,7 +444,7 @@ each_part (MuMsg *msg, MuMsgPart *part, GSList **attlist)
 	att = g_strdup_printf (
 		"(:file-name \"%s\" :mime-type \"%s/%s\")",
 		cachefile, part->type, part->subtype);
-	*attlist = g_slist_append (*attlist, att);
+	pinfo->attlist = g_slist_append (pinfo->attlist, att);
 	g_free (cachefile);
 }
 
@@ -450,27 +457,44 @@ each_part (MuMsg *msg, MuMsgPart *part, GSList **attlist)
  *
  */
 static gchar*
-include_attachments (MuMsg *msg)
+include_attachments (MuMsg *msg, MuMsgOptions opts)
 {
-	GSList *attlist, *cur;
+	GSList  *cur;
 	GString *gstr;
+	PartInfo pinfo;
 
-	attlist = NULL;
-	mu_msg_part_foreach (msg, MU_MSG_OPTION_NONE,
+	pinfo.attlist = NULL;
+	pinfo.opts    = opts;
+	mu_msg_part_foreach (msg, opts,
 			     (MuMsgPartForeachFunc)each_part,
-			     &attlist);
+			     &pinfo);
 
 	gstr = g_string_sized_new (512);
 	gstr = g_string_append_c (gstr, '(');
-	for (cur = attlist; cur; cur = g_slist_next (cur))
+	for (cur = pinfo.attlist; cur; cur = g_slist_next (cur))
 		g_string_append (gstr, (gchar*)cur->data);
 	gstr = g_string_append_c (gstr, ')');
 
-	mu_str_free_list (attlist);
+	mu_str_free_list (pinfo.attlist);
 
 	return g_string_free (gstr, FALSE);
 }
 
+static MuMsgOptions
+get_encrypted_msg_opts (GHashTable *args)
+{
+	MuMsgOptions opts;
+
+	opts = MU_MSG_OPTION_NONE;
+
+	if (get_bool_from_args (args, "use-agent", FALSE, NULL))
+		opts |= MU_MSG_OPTION_USE_AGENT;
+	if (get_bool_from_args (args, "extract-encrypted", FALSE, NULL))
+		opts |= MU_MSG_OPTION_DECRYPT;
+
+	return opts;
+}
+
 enum { NEW, REPLY, FORWARD, EDIT, INVALID_TYPE };
 static unsigned
 compose_type (const char *typestr)
@@ -505,6 +529,9 @@ cmd_compose (ServerContext *ctx, GHashTable *args, GError **err)
 	const gchar *typestr;
 	char *sexp, *atts;
 	unsigned ctype;
+	MuMsgOptions opts;
+
+	opts = get_encrypted_msg_opts (args);
 
 	GET_STRING_OR_ERROR_RETURN (args, "type", &typestr, err);
 
@@ -523,9 +550,8 @@ cmd_compose (ServerContext *ctx, GHashTable *args, GError **err)
 			print_and_clear_g_error (err);
 			return MU_OK;
 		}
-		sexp = mu_msg_to_sexp (msg, atoi(docidstr), NULL,
-				       MU_MSG_OPTION_NONE);
-		atts = (ctype == FORWARD) ? include_attachments (msg) : NULL;
+		sexp = mu_msg_to_sexp (msg, atoi(docidstr), NULL, opts);
+		atts = (ctype == FORWARD) ? include_attachments (msg, opts) : NULL;
 		mu_msg_unref (msg);
 	} else
 		atts = sexp = NULL;
@@ -675,8 +701,8 @@ print_sexps (MuMsgIter *iter, unsigned maxnum)
 
 
 static MuError
-save_part (MuMsg *msg, unsigned docid,
-	   unsigned index, GHashTable *args, GError **err)
+save_part (MuMsg *msg, unsigned docid, unsigned index,
+           MuMsgOptions opts, GHashTable *args, GError **err)
 {
 	gboolean rv;
 	const gchar *path;
@@ -684,7 +710,7 @@ save_part (MuMsg *msg, unsigned docid,
 
 	GET_STRING_OR_ERROR_RETURN (args, "path", &path, err);
 
-	rv = mu_msg_part_save (msg, MU_MSG_OPTION_OVERWRITE,
+	rv = mu_msg_part_save (msg, opts | MU_MSG_OPTION_OVERWRITE,
 			       path, index, err);
 	if (!rv) {
 		print_and_clear_g_error (err);
@@ -701,17 +727,17 @@ save_part (MuMsg *msg, unsigned docid,
 
 
 static MuError
-open_part (MuMsg *msg, unsigned docid, unsigned index, GError **err)
+open_part (MuMsg *msg, unsigned docid, unsigned index,
+           MuMsgOptions opts, GError **err)
 {
 	char *targetpath;
 	gboolean rv;
 
-	targetpath = mu_msg_part_get_cache_path (msg,MU_MSG_OPTION_NONE,
-						 index, err);
+	targetpath = mu_msg_part_get_cache_path (msg, opts, index, err);
 	if (!targetpath)
 		return print_and_clear_g_error (err);
 
-	rv = mu_msg_part_save (msg, MU_MSG_OPTION_USE_EXISTING,
+	rv = mu_msg_part_save (msg, opts | MU_MSG_OPTION_USE_EXISTING,
 			       targetpath, index, err);
 	if (!rv) {
 		print_and_clear_g_error (err);
@@ -736,8 +762,8 @@ leave:
 
 
 static MuError
-temp_part (MuMsg *msg, unsigned docid, unsigned index, GHashTable *args,
-	   GError **err)
+temp_part (MuMsg *msg, unsigned docid, unsigned index,
+           MuMsgOptions opts, GHashTable *args, GError **err)
 {
 	const char *what, *param;
 	char *path;
@@ -745,11 +771,10 @@ temp_part (MuMsg *msg, unsigned docid, unsigned index, GHashTable *args,
 	GET_STRING_OR_ERROR_RETURN (args, "what", &what, err);
 	param = get_string_from_args (args, "param", TRUE, NULL);
 
-	path = mu_msg_part_get_cache_path (msg, MU_MSG_OPTION_NONE,
-					   index, err);
+	path = mu_msg_part_get_cache_path (msg, opts, index, err);
 	if (!path)
 		print_and_clear_g_error (err);
-	else if (!mu_msg_part_save (msg, MU_MSG_OPTION_USE_EXISTING,
+	else if (!mu_msg_part_save (msg, opts | MU_MSG_OPTION_USE_EXISTING,
 				    path, index, err))
 		print_and_clear_g_error (err);
 	else {
@@ -777,7 +802,6 @@ temp_part (MuMsg *msg, unsigned docid, unsigned index, GHashTable *args,
 	return MU_OK;
 }
 
-
 enum { SAVE, OPEN, TEMP, INVALID_ACTION };
 static int
 action_type (const char *actionstr)
@@ -799,8 +823,10 @@ cmd_extract (ServerContext *ctx, GHashTable *args, GError **err)
 	MuMsg *msg;
 	int docid, index, action;
 	MuError rv;
+	MuMsgOptions opts;
 	const char* actionstr, *indexstr;
 
+	opts = get_encrypted_msg_opts (args);
 	rv = MU_ERROR;
 
 	/* read parameters */
@@ -824,9 +850,9 @@ cmd_extract (ServerContext *ctx, GHashTable *args, GError **err)
 	}
 
 	switch (action) {
-	case SAVE: rv = save_part (msg, docid, index, args, err); break;
-	case OPEN: rv = open_part (msg, docid, index, err); break;
-	case TEMP: rv = temp_part (msg, docid, index, args, err); break;
+	case SAVE: rv = save_part (msg, docid, index, opts, args, err); break;
+	case OPEN: rv = open_part (msg, docid, index, opts, err); break;
+	case TEMP: rv = temp_part (msg, docid, index, opts, args, err); break;
 	default: print_error (MU_ERROR_INTERNAL, "unknown action");
 	}
 
