@@ -1,7 +1,7 @@
 /* -*-mode: c++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8-*- */
 
 /*
-** Copyright (C) 2008-2013 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+** Copyright (C) 2008-2015 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -37,7 +37,7 @@ show_version (void)
 {
 	const char* blurb =
 		"mu (mail indexer/searcher) version " VERSION "\n"
-		"Copyright (C) 2008-2013 Dirk-Jan C. Binnema\n"
+		"Copyright (C) 2008-2015 Dirk-Jan C. Binnema\n"
 		"License GPLv3+: GNU GPL version 3 or later "
 		"<http://gnu.org/licenses/gpl.html>.\n"
 		"This is free software: you are free to change "
@@ -64,18 +64,24 @@ handle_error (MuConfig *conf, MuError merr, GError **err)
 			    "try 'mu index --rebuild'\n");
 		break;
 	case MU_ERROR_XAPIAN_IS_EMPTY:
-		g_printerr ("database is empty; try 'mu index'");
+		g_printerr ("database is empty; try 'mu index'\n");
 		break;
 	case MU_ERROR_IN_PARAMETERS:
 		if (conf && mu_config_cmd_is_valid(conf->cmd))
 			mu_config_show_help (conf->cmd);
 		break;
+	case MU_ERROR_SCRIPT_NOT_FOUND:
+		g_printerr ("see the mu manpage for commands, or "
+			    "'mu script' for the scripts\n");
+		break;
 	default:
 		break; /* nothing to do */
 	}
 
 	if (*err)
-		g_printerr ("mu: %s\n", (*err)->message);
+		g_printerr ("mu: %s (%u)\n",
+			    (*err)->message,
+			    (*err)->code);
 }
 
 
@@ -87,7 +93,10 @@ main (int argc, char *argv[])
 	MuConfig *conf;
 
 	setlocale (LC_ALL, "");
+
+#ifndef GLIB_VERSION_2_36
 	g_type_init ();
+#endif /*GLIB_VERSION_2_36*/
 
 	err = NULL;
 	rv  = MU_OK;
