@@ -49,25 +49,39 @@ path."
   :safe 'stringp)
 
 (defcustom mu4e-maildir (expand-file-name "~/Maildir")
-  "The file system path to your Maildir."
+  "The file system path to your Maildir. Must not be a symbolic
+link."
   :type 'directory
   :safe 'stringp
   :group 'mu4e)
 
 (defcustom mu4e-get-mail-command "true"
   "Shell command to run to retrieve new mail.
-Common values are \"offlineimap\" and \"fetchmail\", but you use
-arbitrary shell-commands.
-
-If you set it to \"true\" (the default), the command won't don't
-anything, which is useful if you get your mail without the need to
-explicitly run any scripts, for example when running yout own
-mail-server.
-"
+Common values are \"offlineimap\", \"fetchmail\" and \"mbsync\",
+but you use arbitrary shell-commands. If you set it to
+\"true\" (the default), the command won't don't anything, which is
+useful if you get your mail without the need to explicitly run any
+scripts, for example when running your own mail-server."
   :type 'string
   :group 'mu4e
   :safe 'stringp)
 
+(defcustom mu4e-index-update-error-warning t
+  "Whether to display warnings when we the retrieval process (as
+  per `mu4e-get-mail-command') finished with a non-zero exit code."
+  :type 'boolean
+  :group 'mu4e
+  :safe 'booleanp)
+
+(defcustom mu4e-index-update-error-continue t
+  "Whether to continue with indexing when we the retrieval
+  process (as per `mu4e-get-mail-command') finished with a non-zero
+  exit code."
+  :type 'boolean
+  :group 'mu4e
+  :safe 'booleanp)
+
+
 (defcustom mu4e-update-interval nil
   "Number of seconds between automatic calls to retrieve mail and
 update the database. If nil, don't update automatically. Note,
@@ -236,7 +250,7 @@ The setting is a symbol:
   "Whether to consider only 'personal' e-mail addresses,
 i.e. addresses from messages where user was explicitly in one of
 the address fields (this excludes mailing list messages). See
-`mu4e-my-email-addresses' and the mu-index manpage for details for
+`mu4e-user-mail-address-list' and the mu-index manpage for details for
 details (in particular, how to define your own e-mail addresses)."
   :type 'boolean
   :group 'mu4e-compose)
@@ -256,8 +270,8 @@ Set to nil to not have any time-based restriction."
 ;;; function for mapping a contact onto the canonical one.
 (defun mu4e-contact-identity (contact)
   "This returns the name and the mail-address of a contact.
-It's used as an identity function for converting contacts to their
-canonical counterpart."
+It is used as the identity function for converting contacts to
+their canonical counterpart; useful as an example."
     (let ((name (plist-get contact :name))
           (mail (plist-get contact :mail)))
       (list :name name :mail mail)))
@@ -265,7 +279,7 @@ canonical counterpart."
 (defcustom mu4e-contact-rewrite-function nil
   "Function to be used for when processing contacts and rewrite
 them, for example you may use this for correcting typo's, changed
-names and adapting addresses or names to company policies. As as
+names and adapting addresses or names to company policies. As an
 example of this, see `mu4e-contact-identity'."
   :type 'function
   :group 'mu4e-compose)
@@ -434,7 +448,7 @@ I.e. a message with the draft flag set."
   '((t :inherit font-lock-preprocessor-face))
   "Face for the mark in the headers list."
   :group 'mu4e-faces)
- 
+
 (defface mu4e-header-key-face
   '((t :inherit message-header-name-face :bold t))
   "Face for a header key (such as \"Foo\" in \"Subject:\ Foo\")."
@@ -470,6 +484,11 @@ I.e. a message with the draft flag set."
   "Face for a header title in the headers view."
   :group 'mu4e-faces)
 
+(defface mu4e-modeline-face
+  '((t :inherit font-lock-string-face :bold t))
+  "Face for the query view in the mode-line."
+  :group 'mu4e-faces)
+
 (defface mu4e-footer-face
   '((t :inherit font-lock-comment-face))
   "Face for message footers (signatures)."
@@ -619,6 +638,11 @@ mu4e-compose-mode."
 	 :shortname "Sgn"
 	 :help "Check for the cryptographic signature"
 	 :sortable nil))
+     (:decryption .
+       ( :name "Decryption"
+	 :shortname "Dec"
+	 :help "Check the cryptographic decryption status"
+	 :sortable nil))
      (:size .
        ( :name "Size"
 	 :shortname "Size"
@@ -634,6 +658,11 @@ mu4e-compose-mode."
 	 :shortname "Tags"
 	 :help "Tags for the message"
 	 :sortable nil))
+     (:thread-subject .
+       ( :name "Subject"
+	 :shortname "Subject"
+	 :help "Subject of the thread"
+	 :sortable :subject))
      (:to .
        ( :name "To"
 	 :shortname "T"
